@@ -47,7 +47,8 @@
 #include <KoColorSpaceTraits.h>
 #include <KoCompositeOpAlphaDarken.h>
 #include <KoCompositeOpOver.h>
-#include "KoOptimizedCompositeOpFactory.h"
+#include <KoOptimizedCompositeOpFactory.h>
+#include <KoAlphaDarkenParamsWrapper.h>
 
 // for posix_memalign()
 #include <stdlib.h>
@@ -189,14 +190,14 @@ void generateDataLine(uint seed, int numPixels, quint8 *srcPixels, quint8 *dstPi
 void printData(int numPixels, quint8 *srcPixels, quint8 *dstPixels, quint8 *mask)
 {
     for (int i = 0; i < numPixels; i++) {
-        dbgKrita << "Src: "
+        qDebug() << "Src: "
                  << srcPixels[i*4] << "\t"
                  << srcPixels[i*4+1] << "\t"
                  << srcPixels[i*4+2] << "\t"
                  << srcPixels[i*4+3] << "\t"
                  << "Msk:" << mask[i];
 
-        dbgKrita << "Dst: "
+        qDebug() << "Dst: "
                  << dstPixels[i*4] << "\t"
                  << dstPixels[i*4+1] << "\t"
                  << dstPixels[i*4+2] << "\t"
@@ -227,7 +228,7 @@ QVector<Tile> generateTiles(int size,
     QVector<Tile> tiles(size);
 
 #ifdef HAVE_VC
-    const int vecSize = Vc::float_v::Size;
+    const int vecSize = Vc::float_v::size();
 #else
     const int vecSize = 1;
 #endif
@@ -300,19 +301,19 @@ bool compareTwoOpsPixels(QVector<Tile> &tiles, channel_type prec) {
 
     for (int i = 0; i < numPixels; i++) {
         if (!comparePixels<channel_type>(dst1, dst2, prec)) {
-            dbgKrita << "Wrong result:" << i;
-            dbgKrita << "Act: " << dst1[0] << dst1[1] << dst1[2] << dst1[3];
-            dbgKrita << "Exp: " << dst2[0] << dst2[1] << dst2[2] << dst2[3];
-            dbgKrita << "Dif: " << dst1[0] - dst2[0] << dst1[1] - dst2[1] << dst1[2] - dst2[2] << dst1[3] - dst2[3];
+            qDebug() << "Wrong result:" << i;
+            qDebug() << "Act: " << dst1[0] << dst1[1] << dst1[2] << dst1[3];
+            qDebug() << "Exp: " << dst2[0] << dst2[1] << dst2[2] << dst2[3];
+            qDebug() << "Dif: " << dst1[0] - dst2[0] << dst1[1] - dst2[1] << dst1[2] - dst2[2] << dst1[3] - dst2[3];
 
             channel_type *s1 = src1 + 4 * i;
             channel_type *s2 = src2 + 4 * i;
 
-            dbgKrita << "SrcA:" << s1[0] << s1[1] << s1[2] << s1[3];
-            dbgKrita << "SrcE:" << s2[0] << s2[1] << s2[2] << s2[3];
+            qDebug() << "SrcA:" << s1[0] << s1[1] << s1[2] << s1[3];
+            qDebug() << "SrcE:" << s2[0] << s2[1] << s2[2] << s2[3];
 
-            dbgKrita << "MskA:" << tiles[0].mask[i];
-            dbgKrita << "MskE:" << tiles[1].mask[i];
+            qDebug() << "MskA:" << tiles[0].mask[i];
+            qDebug() << "MskE:" << tiles[1].mask[i];
 
             return false;
         }
@@ -431,14 +432,14 @@ void benchmarkCompositeOp(const KoCompositeOp *op,
         op->composite(params);
     }
 
-    dbgKrita << testName << "RESULT:" << timer.elapsed() << "msec";
+    qDebug() << testName << "RESULT:" << timer.elapsed() << "msec";
 
     freeTiles(tiles, srcAlignmentShift, dstAlignmentShift);
 }
 
 void benchmarkCompositeOp(const KoCompositeOp *op, const QString &postfix)
 {
-    dbgKrita << "Testing Composite Op:" << op->id() << "(" << postfix << ")";
+    qDebug() << "Testing Composite Op:" << op->id() << "(" << postfix << ")";
 
     benchmarkCompositeOp(op, true, 0.5, 0.3, 0, 0, ALPHA_RANDOM, ALPHA_RANDOM);
     benchmarkCompositeOp(op, true, 0.5, 0.3, 8, 0, ALPHA_RANDOM, ALPHA_RANDOM);
@@ -472,7 +473,7 @@ void checkRounding(qreal opacity, qreal flow, qreal averageOpacity = -1, quint32
     QVector<Tile> tiles =
         generateTiles(2, 0, 0, ALPHA_RANDOM, ALPHA_RANDOM, pixelSize);
 
-    const int vecSize = Vc::float_v::Size;
+    const int vecSize = Vc::float_v::size();
 
     const int numBlocks = numPixels / vecSize;
 
@@ -494,21 +495,21 @@ void checkRounding(qreal opacity, qreal flow, qreal averageOpacity = -1, quint32
     }
 
     params.channelFlags = QBitArray();
-    typename Compositor::OptionalParams optionalParams(params);
+    typename Compositor::ParamsWrapper paramsWrapper(params);
 
     // The error count is needed as 38.5 gets rounded to 38 instead of 39 in the vc version.
     int errorcount = 0;
     for (int i = 0; i < numBlocks; i++) {
-        Compositor::template compositeVector<true,true, VC_IMPL>(src1, dst1, msk1, params.opacity, optionalParams);
+        Compositor::template compositeVector<true,true, Vc::CurrentImplementation::current()>(src1, dst1, msk1, params.opacity, paramsWrapper);
         for (int j = 0; j < vecSize; j++) {
 
             //if (8 * i + j == 7080) {
-            //    dbgKrita << "src: " << src2[0] << src2[1] << src2[2] << src2[3];
-            //    dbgKrita << "dst: " << dst2[0] << dst2[1] << dst2[2] << dst2[3];
-            //    dbgKrita << "msk:" << msk2[0];
+            //    qDebug() << "src: " << src2[0] << src2[1] << src2[2] << src2[3];
+            //    qDebug() << "dst: " << dst2[0] << dst2[1] << dst2[2] << dst2[3];
+            //    qDebug() << "msk:" << msk2[0];
             //}
 
-            Compositor::template compositeOnePixelScalar<true, VC_IMPL>(src2, dst2, msk2, params.opacity, optionalParams);
+            Compositor::template compositeOnePixelScalar<true, Vc::CurrentImplementation::current()>(src2, dst2, msk2, params.opacity, paramsWrapper);
 
             bool compareResult = true;
             if (pixelSize == 4) {
@@ -529,12 +530,12 @@ void checkRounding(qreal opacity, qreal flow, qreal averageOpacity = -1, quint32
             }
 
             if(!compareResult || errorcount > 1) {
-                dbgKrita << "Wrong rounding in pixel:" << 8 * i + j;
-                dbgKrita << "Vector version: " << dst1[0] << dst1[1] << dst1[2] << dst1[3];
-                dbgKrita << "Scalar version: " << dst2[0] << dst2[1] << dst2[2] << dst2[3];
+                qDebug() << "Wrong rounding in pixel:" << 8 * i + j;
+                qDebug() << "Vector version: " << dst1[0] << dst1[1] << dst1[2] << dst1[3];
+                qDebug() << "Scalar version: " << dst2[0] << dst2[1] << dst2[2] << dst2[3];
 
-                dbgKrita << "src:" << src1[0] << src1[1] << src1[2] << src1[3];
-                dbgKrita << "msk:" << msk1[0];
+                qDebug() << "src:" << src1[0] << src1[1] << src1[2] << src1[3];
+                qDebug() << "msk:" << msk1[0];
 
                 QFAIL("Wrong rounding");
             }
@@ -557,35 +558,35 @@ void checkRounding(qreal opacity, qreal flow, qreal averageOpacity = -1, quint32
 void KisCompositionBenchmark::checkRoundingAlphaDarken_05_03()
 {
 #ifdef HAVE_VC
-    checkRounding<AlphaDarkenCompositor32<quint8, quint32> >(0.5,0.3);
+    checkRounding<AlphaDarkenCompositor32<quint8, quint32, KoAlphaDarkenParamsWrapperCreamy> >(0.5,0.3);
 #endif
 }
 
 void KisCompositionBenchmark::checkRoundingAlphaDarken_05_05()
 {
 #ifdef HAVE_VC
-    checkRounding<AlphaDarkenCompositor32<quint8, quint32> >(0.5,0.5);
+    checkRounding<AlphaDarkenCompositor32<quint8, quint32, KoAlphaDarkenParamsWrapperCreamy> >(0.5,0.5);
 #endif
 }
 
 void KisCompositionBenchmark::checkRoundingAlphaDarken_05_07()
 {
 #ifdef HAVE_VC
-    checkRounding<AlphaDarkenCompositor32<quint8, quint32> >(0.5,0.7);
+    checkRounding<AlphaDarkenCompositor32<quint8, quint32, KoAlphaDarkenParamsWrapperCreamy> >(0.5,0.7);
 #endif
 }
 
 void KisCompositionBenchmark::checkRoundingAlphaDarken_05_10()
 {
 #ifdef HAVE_VC
-    checkRounding<AlphaDarkenCompositor32<quint8, quint32> >(0.5,1.0);
+    checkRounding<AlphaDarkenCompositor32<quint8, quint32, KoAlphaDarkenParamsWrapperCreamy> >(0.5,1.0);
 #endif
 }
 
 void KisCompositionBenchmark::checkRoundingAlphaDarken_05_10_08()
 {
 #ifdef HAVE_VC
-    checkRounding<AlphaDarkenCompositor32<quint8, quint32> >(0.5,1.0,0.8);
+    checkRounding<AlphaDarkenCompositor32<quint8, quint32, KoAlphaDarkenParamsWrapperCreamy> >(0.5,1.0,0.8);
 #endif
 }
 
@@ -641,8 +642,8 @@ void KisCompositionBenchmark::checkRoundingOverRgbaF32()
 void KisCompositionBenchmark::compareAlphaDarkenOps()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->rgb8();
-    KoCompositeOp *opAct = KoOptimizedCompositeOpFactory::createAlphaDarkenOp32(cs);
-    KoCompositeOp *opExp = new KoCompositeOpAlphaDarken<KoBgrU8Traits>(cs);
+    KoCompositeOp *opAct = KoOptimizedCompositeOpFactory::createAlphaDarkenOpCreamy32(cs);
+    KoCompositeOp *opExp = new KoCompositeOpAlphaDarken<KoBgrU8Traits, KoAlphaDarkenParamsWrapperCreamy>(cs);
 
     QVERIFY(compareTwoOps(true, opAct, opExp));
 
@@ -653,8 +654,8 @@ void KisCompositionBenchmark::compareAlphaDarkenOps()
 void KisCompositionBenchmark::compareRgbF32AlphaDarkenOps()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace("RGBA", "F32", "");
-    KoCompositeOp *opAct = KoOptimizedCompositeOpFactory::createAlphaDarkenOp128(cs);
-    KoCompositeOp *opExp = new KoCompositeOpAlphaDarken<KoRgbF32Traits>(cs);
+    KoCompositeOp *opAct = KoOptimizedCompositeOpFactory::createAlphaDarkenOpCreamy128(cs);
+    KoCompositeOp *opExp = new KoCompositeOpAlphaDarken<KoRgbF32Traits, KoAlphaDarkenParamsWrapperCreamy>(cs);
 
     QVERIFY(compareTwoOps(true, opAct, opExp));
 
@@ -665,8 +666,8 @@ void KisCompositionBenchmark::compareRgbF32AlphaDarkenOps()
 void KisCompositionBenchmark::compareAlphaDarkenOpsNoMask()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->rgb8();
-    KoCompositeOp *opAct = KoOptimizedCompositeOpFactory::createAlphaDarkenOp32(cs);
-    KoCompositeOp *opExp = new KoCompositeOpAlphaDarken<KoBgrU8Traits>(cs);
+    KoCompositeOp *opAct = KoOptimizedCompositeOpFactory::createAlphaDarkenOpCreamy32(cs);
+    KoCompositeOp *opExp = new KoCompositeOpAlphaDarken<KoBgrU8Traits, KoAlphaDarkenParamsWrapperCreamy>(cs);
 
     QVERIFY(compareTwoOps(false, opAct, opExp));
 
@@ -713,7 +714,7 @@ void KisCompositionBenchmark::compareRgbF32OverOps()
 void KisCompositionBenchmark::testRgb8CompositeAlphaDarkenLegacy()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->rgb8();
-    KoCompositeOp *op = new KoCompositeOpAlphaDarken<KoBgrU8Traits>(cs);
+    KoCompositeOp *op = new KoCompositeOpAlphaDarken<KoBgrU8Traits, KoAlphaDarkenParamsWrapperCreamy>(cs);
     benchmarkCompositeOp(op, "Legacy");
     delete op;
 }
@@ -721,7 +722,7 @@ void KisCompositionBenchmark::testRgb8CompositeAlphaDarkenLegacy()
 void KisCompositionBenchmark::testRgb8CompositeAlphaDarkenOptimized()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->rgb8();
-    KoCompositeOp *op = KoOptimizedCompositeOpFactory::createAlphaDarkenOp32(cs);
+    KoCompositeOp *op = KoOptimizedCompositeOpFactory::createAlphaDarkenOpCreamy32(cs);
     benchmarkCompositeOp(op, "Optimized");
     delete op;
 }
@@ -745,7 +746,7 @@ void KisCompositionBenchmark::testRgb8CompositeOverOptimized()
 void KisCompositionBenchmark::testRgbF32CompositeAlphaDarkenLegacy()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace("RGBA", "F32", "");
-    KoCompositeOp *op = new KoCompositeOpAlphaDarken<KoRgbF32Traits>(cs);
+    KoCompositeOp *op = new KoCompositeOpAlphaDarken<KoRgbF32Traits, KoAlphaDarkenParamsWrapperCreamy>(cs);
     benchmarkCompositeOp(op, "Legacy");
     delete op;
 }
@@ -753,7 +754,7 @@ void KisCompositionBenchmark::testRgbF32CompositeAlphaDarkenLegacy()
 void KisCompositionBenchmark::testRgbF32CompositeAlphaDarkenOptimized()
 {
     const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace("RGBA", "F32", "");
-    KoCompositeOp *op = KoOptimizedCompositeOpFactory::createAlphaDarkenOp128(cs);
+    KoCompositeOp *op = KoOptimizedCompositeOpFactory::createAlphaDarkenOpCreamy128(cs);
     benchmarkCompositeOp(op, "Optimized");
     delete op;
 }
@@ -810,7 +811,7 @@ void KisCompositionBenchmark::benchmarkMemcpy()
 }
 
 #ifdef HAVE_VC
-    const int vecSize = Vc::float_v::Size;
+    const int vecSize = Vc::float_v::size();
     const size_t uint8VecAlignment = qMax(vecSize * sizeof(quint8), sizeof(void*));
     const size_t uint32VecAlignment = qMax(vecSize * sizeof(quint32), sizeof(void*));
     const size_t floatVecAlignment = qMax(vecSize * sizeof(float), sizeof(void*));
@@ -819,6 +820,8 @@ void KisCompositionBenchmark::benchmarkMemcpy()
 void KisCompositionBenchmark::benchmarkUintFloat()
 {
 #ifdef HAVE_VC
+    using uint_v = Vc::SimdArray<unsigned int, Vc::float_v::size()>;
+
     const int dataSize = 4096;
     void *ptr = 0;
     int error = MEMALIGN_ALLOC(&ptr, uint8VecAlignment, dataSize);
@@ -833,10 +836,10 @@ void KisCompositionBenchmark::benchmarkUintFloat()
     float *fData = (float*)ptr;
 
     QBENCHMARK {
-        for (int i = 0; i < dataSize; i += Vc::float_v::Size) {
+        for (int i = 0; i < dataSize; i += Vc::float_v::size()) {
             // convert uint -> float directly, this causes
             // static_cast helper be called
-            Vc::float_v b(Vc::uint_v(iData + i));
+            Vc::float_v b(uint_v(iData + i));
             b.store(fData + i);
         }
     }
@@ -849,6 +852,9 @@ void KisCompositionBenchmark::benchmarkUintFloat()
 void KisCompositionBenchmark::benchmarkUintIntFloat()
 {
 #ifdef HAVE_VC
+    using int_v = Vc::SimdArray<int, Vc::float_v::size()>;
+    using uint_v = Vc::SimdArray<unsigned int, Vc::float_v::size()>;
+
     const int dataSize = 4096;
     void *ptr = 0;
     int error = MEMALIGN_ALLOC(&ptr, uint8VecAlignment, dataSize);
@@ -863,10 +869,10 @@ void KisCompositionBenchmark::benchmarkUintIntFloat()
     float *fData = (float*)ptr;
 
     QBENCHMARK {
-        for (int i = 0; i < dataSize; i += Vc::float_v::Size) {
+        for (int i = 0; i < dataSize; i += Vc::float_v::size()) {
             // convert uint->int->float, that avoids special sign
             // treating, and gives 2.6 times speedup
-            Vc::float_v b(Vc::int_v(Vc::uint_v(iData + i)));
+            Vc::float_v b(int_v(uint_v(iData + i)));
             b.store(fData + i);
         }
     }
@@ -879,6 +885,8 @@ void KisCompositionBenchmark::benchmarkUintIntFloat()
 void KisCompositionBenchmark::benchmarkFloatUint()
 {
 #ifdef HAVE_VC
+    using uint_v = Vc::SimdArray<unsigned int, Vc::float_v::size()>;
+
     const int dataSize = 4096;
     void *ptr = 0;
     int error = MEMALIGN_ALLOC(&ptr, uint32VecAlignment, dataSize * sizeof(quint32));
@@ -893,9 +901,9 @@ void KisCompositionBenchmark::benchmarkFloatUint()
     float *fData = (float*)ptr;
 
     QBENCHMARK {
-        for (int i = 0; i < dataSize; i += Vc::float_v::Size) {
+        for (int i = 0; i < dataSize; i += Vc::float_v::size()) {
             // conversion float -> uint
-            Vc::uint_v b(Vc::float_v(fData + i));
+            uint_v b(Vc::float_v(fData + i));
 
             b.store(iData + i);
         }
@@ -909,6 +917,9 @@ void KisCompositionBenchmark::benchmarkFloatUint()
 void KisCompositionBenchmark::benchmarkFloatIntUint()
 {
 #ifdef HAVE_VC
+    using int_v = Vc::SimdArray<int, Vc::float_v::size()>;
+    using uint_v = Vc::SimdArray<unsigned int, Vc::float_v::size()>;
+
     const int dataSize = 4096;
     void *ptr = 0;
     int error = MEMALIGN_ALLOC(&ptr, uint32VecAlignment, dataSize * sizeof(quint32));
@@ -923,9 +934,9 @@ void KisCompositionBenchmark::benchmarkFloatIntUint()
     float *fData = (float*)ptr;
 
     QBENCHMARK {
-        for (int i = 0; i < dataSize; i += Vc::float_v::Size) {
+        for (int i = 0; i < dataSize; i += Vc::float_v::size()) {
             // conversion float -> int -> uint
-            Vc::uint_v b(Vc::int_v(Vc::float_v(fData + i)));
+            uint_v b(int_v(Vc::float_v(fData + i)));
 
             b.store(iData + i);
         }
