@@ -59,7 +59,6 @@
 #include "kis_command_utils.h"
 #include "kis_pointer_utils.h"
 
-
 #include <KoIcon.h>
 
 #include <QMenu>
@@ -108,82 +107,48 @@ struct KoPathTool::PathSegment {
 };
 
 KoPathTool::KoPathTool(KoCanvasBase *canvas)
-        : KoToolBase(canvas)
-        , m_pointSelection(this)
-        , m_activeHandle(0)
-        , m_handleRadius(3)
-        , m_activeSegment(0)
-        , m_currentStrategy(0)
-        , m_activatedTemporarily(false)
+    : KoToolBase(canvas)
+    , m_pointSelection(this)
+    , m_activeHandle(0)
+    , m_handleRadius(3)
+    , m_activeSegment(0)
+    , m_currentStrategy(0)
+    , m_activatedTemporarily(false)
 {
-    QActionGroup *points = new QActionGroup(this);
+    m_points = new QActionGroup(this);
     // m_pointTypeGroup->setExclusive(true);
-    KisActionRegistry *actionRegistry = KisActionRegistry::instance();
-    m_actionPathPointCorner = actionRegistry->makeQAction("pathpoint-corner", this);
-    addAction("pathpoint-corner", m_actionPathPointCorner);
-    m_actionPathPointCorner->setData(KoPathPointTypeCommand::Corner);
-    points->addAction(m_actionPathPointCorner);
 
-    m_actionPathPointSmooth = actionRegistry->makeQAction("pathpoint-smooth", this);
-    addAction("pathpoint-smooth", m_actionPathPointSmooth);
-    m_actionPathPointSmooth->setData(KoPathPointTypeCommand::Smooth);
-    points->addAction(m_actionPathPointSmooth);
+    m_actionPathPointCorner = action("pathpoint-corner");
+    if (m_actionPathPointCorner) {
+        m_actionPathPointCorner->setData(KoPathPointTypeCommand::Corner);
+        m_points->addAction(m_actionPathPointCorner);
+    }
+    m_actionPathPointSmooth = action("pathpoint-smooth");
+    if (m_actionPathPointSmooth) {
+        m_actionPathPointSmooth->setData(KoPathPointTypeCommand::Smooth);
+        m_points->addAction(m_actionPathPointSmooth);
+    }
 
-    m_actionPathPointSymmetric = actionRegistry->makeQAction("pathpoint-symmetric", this);
-    addAction("pathpoint-symmetric", m_actionPathPointSymmetric);
-    m_actionPathPointSymmetric->setData(KoPathPointTypeCommand::Symmetric);
-    points->addAction(m_actionPathPointSymmetric);
+    m_actionPathPointSymmetric = action("pathpoint-symmetric");
+    if (m_actionPathPointSymmetric) {
+        m_actionPathPointSymmetric->setData(KoPathPointTypeCommand::Symmetric);
+        m_points->addAction(m_actionPathPointSymmetric);
+    }
 
-    m_actionCurvePoint = actionRegistry->makeQAction("pathpoint-curve", this);
-    addAction("pathpoint-curve", m_actionCurvePoint);
-    connect(m_actionCurvePoint, SIGNAL(triggered()), this, SLOT(pointToCurve()));
-
-    m_actionLinePoint = actionRegistry->makeQAction("pathpoint-line", this);
-    addAction("pathpoint-line", m_actionLinePoint);
-    connect(m_actionLinePoint, SIGNAL(triggered()), this, SLOT(pointToLine()));
-
-    m_actionLineSegment = actionRegistry->makeQAction("pathsegment-line", this);
-    addAction("pathsegment-line", m_actionLineSegment);
-    connect(m_actionLineSegment, SIGNAL(triggered()), this, SLOT(segmentToLine()));
-
-    m_actionCurveSegment = actionRegistry->makeQAction("pathsegment-curve", this);
-    addAction("pathsegment-curve", m_actionCurveSegment);
-    connect(m_actionCurveSegment, SIGNAL(triggered()), this, SLOT(segmentToCurve()));
-
-    m_actionAddPoint = actionRegistry->makeQAction("pathpoint-insert", this);
-    addAction("pathpoint-insert", m_actionAddPoint);
-    connect(m_actionAddPoint, SIGNAL(triggered()), this, SLOT(insertPoints()));
-
-    m_actionRemovePoint = actionRegistry->makeQAction("pathpoint-remove", this);
-    addAction("pathpoint-remove", m_actionRemovePoint);
-    connect(m_actionRemovePoint, SIGNAL(triggered()), this, SLOT(removePoints()));
-
-    m_actionBreakPoint = actionRegistry->makeQAction("path-break-point", this);
-    addAction("path-break-point", m_actionBreakPoint);
-    connect(m_actionBreakPoint, SIGNAL(triggered()), this, SLOT(breakAtPoint()));
-
-    m_actionBreakSegment = actionRegistry->makeQAction("path-break-segment", this);
-    addAction("path-break-segment", m_actionBreakSegment);
-    connect(m_actionBreakSegment, SIGNAL(triggered()), this, SLOT(breakAtSegment()));
-
-    m_actionJoinSegment = actionRegistry->makeQAction("pathpoint-join", this);
-    addAction("pathpoint-join", m_actionJoinSegment);
-    connect(m_actionJoinSegment, SIGNAL(triggered()), this, SLOT(joinPoints()));
-
-    m_actionMergePoints = actionRegistry->makeQAction("pathpoint-merge", this);
-    addAction("pathpoint-merge", m_actionMergePoints);
-    connect(m_actionMergePoints, SIGNAL(triggered()), this, SLOT(mergePoints()));
-
-    m_actionConvertToPath = actionRegistry->makeQAction("convert-to-path", this);
-    addAction("convert-to-path", m_actionConvertToPath);
-    connect(m_actionConvertToPath, SIGNAL(triggered()), this, SLOT(convertToPath()));
+    m_actionCurvePoint = action("pathpoint-curve");
+    m_actionLinePoint = action("pathpoint-line");
+    m_actionLineSegment = action("pathsegment-line");
+    m_actionCurveSegment = action("pathsegment-curve");
+    m_actionAddPoint = action("pathpoint-insert");
+    m_actionRemovePoint = action("pathpoint-remove");
+    m_actionBreakPoint = action("path-break-point");
+    m_actionBreakSegment = action("path-break-segment");
+    m_actionJoinSegment = action("pathpoint-join");
+    m_actionMergePoints = action("pathpoint-merge");
+    m_actionConvertToPath = action("convert-to-path");
 
     m_contextMenu.reset(new QMenu());
 
-
-    connect(points, SIGNAL(triggered(QAction*)), this, SLOT(pointTypeChanged(QAction*)));
-    connect(&m_pointSelection, SIGNAL(selectionChanged()), this, SLOT(pointSelectionChanged()));
-
     QBitmap b = QBitmap::fromData(QSize(16, 16), needle_bits);
     QBitmap m = b.createHeuristicMask(false);
 
@@ -232,8 +197,8 @@ void KoPathTool::pointTypeChanged(QAction *type)
         }
 
         KUndo2Command *command =
-            new KoPathPointTypeCommand(selectedPoints,
-                                       static_cast<KoPathPointTypeCommand::PointType>(type->data().toInt()));
+                new KoPathPointTypeCommand(selectedPoints,
+                                           static_cast<KoPathPointTypeCommand::PointType>(type->data().toInt()));
 
         if (initialConversionCommand) {
             using namespace KisCommandUtils;
@@ -391,7 +356,8 @@ void KoPathTool::convertToPath()
         const QList<KoShape*> oldSelectedShapes = implicitCastList<KoShape*>(textShapes);
 
 
-        new KoKeepShapesSelectedCommand(oldSelectedShapes, {}, canvas()->selectedShapesProxy(), false, cmd);
+        new KoKeepShapesSelectedCommand(oldSelectedShapes, {}, canvas()->selectedShapesProxy(),
+                                        KisCommandUtils::FlipFlopCommand::State::INITIALIZING, cmd);
 
         QList<KoShape*> newSelectedShapes;
         Q_FOREACH (KoSvgTextShape *shape, textShapes) {
@@ -413,7 +379,8 @@ void KoPathTool::convertToPath()
 
         canvas()->shapeController()->removeShapes(oldSelectedShapes, cmd);
 
-        new KoKeepShapesSelectedCommand({}, newSelectedShapes, canvas()->selectedShapesProxy(), true, cmd);
+        new KoKeepShapesSelectedCommand({}, newSelectedShapes, canvas()->selectedShapesProxy(),
+                                        KisCommandUtils::FlipFlopCommand::State::FINALIZING, cmd);
 
         canvas()->addCommand(cmd);
     }
@@ -511,7 +478,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
 
     Q_FOREACH (KoPathShape *shape, m_pointSelection.selectedShapes()) {
         KisHandlePainterHelper helper =
-            KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
         helper.setHandleStyle(KisHandleStyle::primarySelection());
 
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
@@ -554,7 +521,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
             KIS_SAFE_ASSERT_RECOVER_RETURN(segment.isValid());
 
             KisHandlePainterHelper helper =
-                KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                    KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
             helper.setHandleStyle(KisHandleStyle::secondarySelection());
 
             QPainterPath path;
@@ -678,12 +645,12 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
                 delete m_activeHandle;
 
                 if (KoConnectionShape * connectionShape = dynamic_cast<KoConnectionShape*>(parameterShape)) {
-                    //qDebug() << "handleId" << handleId;
+                    //debugFlake << "handleId" << handleId;
                     m_activeHandle = new ConnectionHandle(this, connectionShape, handleId);
                     m_activeHandle->repaint();
                     return;
                 } else {
-                    //qDebug() << "handleId" << handleId;
+                    //debugFlake << "handleId" << handleId;
                     m_activeHandle = new ParameterHandle(this, parameterShape, handleId);
                     m_activeHandle->repaint();
                     return;
@@ -810,7 +777,6 @@ void KoPathTool::mouseReleaseEvent(KoPointerEvent *event)
 
 void KoPathTool::keyPressEvent(QKeyEvent *event)
 {
-    Q_D(KoToolBase);
     if (m_currentStrategy) {
         switch (event->key()) {
         case Qt::Key_Control:
@@ -885,8 +851,8 @@ void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
     if (!m_activeHandle && m_activeSegment && m_activeSegment->isValid()) {
         QList<KoPathPointData> segments;
         segments.append(
-            KoPathPointData(m_activeSegment->path,
-                            m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart)));
+                    KoPathPointData(m_activeSegment->path,
+                                    m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart)));
 
         KoPathPointInsertCommand *cmd = new KoPathPointInsertCommand(segments, m_activeSegment->positionOnSegment);
         d->canvas->addCommand(cmd);
@@ -911,8 +877,6 @@ void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
 
 KoPathTool::PathSegment* KoPathTool::segmentAtPoint(const QPointF &point)
 {
-    Q_D(KoToolBase);
-
     // the max allowed distance from a segment
     const QRectF grabRoi = handleGrabRect(point);
     const qreal distanceThreshold = 0.5 * KisAlgebra2D::maxDimension(grabRoi);
@@ -976,13 +940,28 @@ void KoPathTool::activate(ToolActivation activation, const QSet<KoShape*> &shape
     m_shapeFillResourceConnector.connectToCanvas(d->canvas);
 
     initializeWithShapes(shapes.toList());
+
+    connect(m_actionCurvePoint, SIGNAL(triggered()), this, SLOT(pointToCurve()), Qt::UniqueConnection);
+    connect(m_actionLinePoint, SIGNAL(triggered()), this, SLOT(pointToLine()), Qt::UniqueConnection);
+    connect(m_actionLineSegment, SIGNAL(triggered()), this, SLOT(segmentToLine()), Qt::UniqueConnection);
+    connect(m_actionCurveSegment, SIGNAL(triggered()), this, SLOT(segmentToCurve()), Qt::UniqueConnection);
+    connect(m_actionAddPoint, SIGNAL(triggered()), this, SLOT(insertPoints()), Qt::UniqueConnection);
+    connect(m_actionRemovePoint, SIGNAL(triggered()), this, SLOT(removePoints()), Qt::UniqueConnection);
+    connect(m_actionBreakPoint, SIGNAL(triggered()), this, SLOT(breakAtPoint()), Qt::UniqueConnection);
+    connect(m_actionBreakSegment, SIGNAL(triggered()), this, SLOT(breakAtSegment()), Qt::UniqueConnection);
+    connect(m_actionJoinSegment, SIGNAL(triggered()), this, SLOT(joinPoints()), Qt::UniqueConnection);
+    connect(m_actionMergePoints, SIGNAL(triggered()), this, SLOT(mergePoints()), Qt::UniqueConnection);
+    connect(m_actionConvertToPath, SIGNAL(triggered()), this, SLOT(convertToPath()), Qt::UniqueConnection);
+    connect(m_points, SIGNAL(triggered(QAction*)), this, SLOT(pointTypeChanged(QAction*)), Qt::UniqueConnection);
+    connect(&m_pointSelection, SIGNAL(selectionChanged()), this, SLOT(pointSelectionChanged()), Qt::UniqueConnection);
+
 }
 
 void KoPathTool::slotSelectionChanged()
 {
     Q_D(KoToolBase);
     QList<KoShape*> shapes =
-        d->canvas->selectedShapesProxy()->selection()->selectedEditableShapesAndDelegates();
+            d->canvas->selectedShapesProxy()->selection()->selectedEditableShapesAndDelegates();
 
     initializeWithShapes(shapes);
 }
@@ -1022,7 +1001,7 @@ void KoPathTool::initializeWithShapes(const QList<KoShape*> shapes)
     }
 
     const QRectF oldBoundingRect =
-        KoShape::boundingRect(implicitCastList<KoShape*>(m_pointSelection.selectedShapes()));
+            KoShape::boundingRect(implicitCastList<KoShape*>(m_pointSelection.selectedShapes()));
 
     if (selectedShapes != m_pointSelection.selectedShapes()) {
         clearActivePointSelectionReferences();
@@ -1049,7 +1028,7 @@ void KoPathTool::updateOptionsWidget()
     Q_FOREACH (KoPathShape *shape, selectedShapes) {
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
         type |= parameterShape && parameterShape->isParametricShape() ?
-                PathToolOptionWidget::ParametricShape : PathToolOptionWidget::PlainPath;
+                    PathToolOptionWidget::ParametricShape : PathToolOptionWidget::PlainPath;
     }
 
     emit singleShapeChanged(selectedShapes.size() == 1 ? selectedShapes.first() : 0);
@@ -1077,15 +1056,15 @@ void KoPathTool::updateActions()
             const int pointIndex = pd.pointIndex.second;
 
             canBreakAtPoint |= pd.pathShape->isClosedSubpath(subpathIndex) ||
-                (pointIndex > 0 && pointIndex < pd.pathShape->subpathPointCount(subpathIndex) - 1);
+                    (pointIndex > 0 && pointIndex < pd.pathShape->subpathPointCount(subpathIndex) - 1);
 
             KoPathPoint *point = pd.pathShape->pointByIndex(pd.pointIndex);
 
             hasNonSmoothPoints |= !(point->properties() & KoPathPoint::IsSmooth);
             hasNonSymmetricPoints |= !(point->properties() & KoPathPoint::IsSymmetric);
             hasNonSplitPoints |=
-                point->properties() & KoPathPoint::IsSymmetric ||
-                point->properties() & KoPathPoint::IsSmooth;
+                    point->properties() & KoPathPoint::IsSymmetric ||
+                    point->properties() & KoPathPoint::IsSmooth;
 
             hasNonLinePoints |= point->activeControlPoint1() || point->activeControlPoint2();
             hasNonCurvePoints |= !point->activeControlPoint1() && !point->activeControlPoint2();
@@ -1150,7 +1129,7 @@ void KoPathTool::updateActions()
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
         KoSvgTextShape *textShape = dynamic_cast<KoSvgTextShape*>(shape);
         if (textShape ||
-            (parameterShape && parameterShape->isParametricShape())) {
+                (parameterShape && parameterShape->isParametricShape())) {
 
             haveConvertibleShapes = true;
             break;
@@ -1175,6 +1154,20 @@ void KoPathTool::deactivate()
     m_currentStrategy = 0;
     d->canvas->snapGuide()->reset();
 
+    disconnect(m_actionCurvePoint, 0, this, 0);
+    disconnect(m_actionLinePoint, 0, this, 0);
+    disconnect(m_actionLineSegment, 0, this, 0);
+    disconnect(m_actionCurveSegment, 0, this, 0);
+    disconnect(m_actionAddPoint, 0, this, 0);
+    disconnect(m_actionRemovePoint, 0, this, 0);
+    disconnect(m_actionBreakPoint, 0, this, 0);
+    disconnect(m_actionBreakSegment, 0, this, 0);
+    disconnect(m_actionJoinSegment, 0, this, 0);
+    disconnect(m_actionMergePoints, 0, this, 0);
+    disconnect(m_actionConvertToPath, 0, this, 0);
+    disconnect(m_points, 0, this, 0);
+    disconnect(&m_pointSelection, 0, this, 0);
+
     KoToolBase::deactivate();
 }
 
