@@ -43,6 +43,8 @@
 #include "kis_mask_projection_plane.h"
 
 #include "kis_raster_keyframe_channel.h"
+#include "KisSafeNodeProjectionStore.h"
+
 
 struct Q_DECL_HIDDEN KisMask::Private {
     Private(KisMask *_q)
@@ -66,16 +68,18 @@ struct Q_DECL_HIDDEN KisMask::Private {
     QScopedPointer<QPoint> deferredSelectionOffset;
 
     KisAbstractProjectionPlaneSP projectionPlane;
-    KisCachedSelection cachedSelection;
+    KisSafeSelectionNodeProjectionStoreSP safeProjection;
 
     void initSelectionImpl(KisSelectionSP copyFrom, KisLayerSP parentLayer, KisPaintDeviceSP copyFromDevice);
 };
 
 KisMask::KisMask(const QString & name)
-        : KisNode()
+        : KisNode(nullptr)
         , m_d(new Private(this))
 {
     setName(name);
+    m_d->safeProjection = new KisSafeSelectionNodeProjectionStore();
+    m_d->safeProjection->setImage(image());
 }
 
 KisMask::KisMask(const KisMask& rhs)
@@ -85,6 +89,8 @@ KisMask::KisMask(const KisMask& rhs)
 {
     setName(rhs.name());
 
+    m_d->safeProjection = new KisSafeSelectionNodeProjectionStore(*rhs.m_d->safeProjection);
+
     if (rhs.m_d->selection) {
         m_d->selection = new KisSelection(*rhs.m_d->selection.data());
         m_d->selection->setParentNode(this);
@@ -99,6 +105,10 @@ KisMask::KisMask(const KisMask& rhs)
 
 KisMask::~KisMask()
 {
+    if (m_d->selection) {
+        m_d->selection->setParentNode(0);
+    }
+
     delete m_d;
 }
 
@@ -109,6 +119,10 @@ void KisMask::setImage(KisImageWSP image)
     if (m_d->selection) {
         m_d->selection->setDefaultBounds(defaultBounds);
     }
+
+    m_d->safeProjection->setImage(image);
+
+    KisNode::setImage(image);
 }
 
 bool KisMask::allowAsChild(KisNodeSP node) const
@@ -170,12 +184,17 @@ void KisMask::Private::initSelectionImpl(KisSelectionSP copyFrom, KisLayerSP par
             delete selection->flatten();
         }
     } else if (copyFromDevice) {
-        selection = new KisSelection(new KisSelectionDefaultBounds(parentPaintDevice, parentLayer->image()));
+        KritaUtils::DeviceCopyMode copyMode =
+            q->inherits("KisFilterMask") || q->inherits("KisTransparencyMask") ?
+            KritaUtils::CopyAllFrames : KritaUtils::CopySnapshot;
 
-        QRect rc(copyFromDevice->extent());
-        KisPainter::copyAreaOptimized(rc.topLeft(), copyFromDevice, selection->pixelSelection(), rc);
-        selection->pixelSelection()->invalidateOutlineCache();
+        selection = new KisSelection(copyFromDevice, copyMode, new KisSelectionDefaultBounds(parentPaintDevice, parentLayer->image()));
 
+        KisPixelSelectionSP pixelSelection = selection->pixelSelection();
+        if (pixelSelection->framesInterface()) {
+            q->addKeyframeChannel(pixelSelection->keyframeChannel());
+            q->enableAnimation();
+        }
     } else {
         selection = new KisSelection(new KisSelectionDefaultBounds(parentPaintDevice, parentLayer->image()));
         selection->pixelSelection()->setDefaultPixel(KoColor(Qt::white, selection->pixelSelection()->colorSpace()));
@@ -197,7 +216,8 @@ KisSelectionSP KisMask::selection() const
 
 KisPaintDeviceSP KisMask::paintDevice() const
 {
-    return selection()->pixelSelection();
+    KisSelectionSP selection = this->selection();
+    return selection ? selection->pixelSelection() : 0;
 }
 
 KisPaintDeviceSP KisMask::original() const
@@ -207,7 +227,15 @@ KisPaintDeviceSP KisMask::original() const
 
 KisPaintDeviceSP KisMask::projection() const
 {
-    return paintDevice();
+    KisPaintDeviceSP originalDevice = original();
+    KisPaintDeviceSP result = originalDevice;
+
+    KisSelectionSP selection = this->selection();
+    if (selection && hasTemporaryTarget()) {
+        result = m_d->safeProjection->getDeviceLazy(selection)->pixelSelection();
+    }
+
+    return result;
 }
 
 KisAbstractProjectionPlaneSP KisMask::projectionPlane() const
@@ -246,34 +274,39 @@ QRect KisMask::decorateRect(KisPaintDeviceSP &src,
     return rc;
 }
 
+bool KisMask::paintsOutsideSelection() const
+{
+    return false;
+}
+
 void KisMask::apply(KisPaintDeviceSP projection, const QRect &applyRect, const QRect &needRect, PositionToFilthy maskPos) const
 {
     if (selection()) {
 
-        m_d->selection->updateProjection(applyRect);
+        flattenSelectionProjection(m_d->selection, applyRect);
 
         KisSelectionSP effectiveSelection = m_d->selection;
-        QRect effectiveExtent;
 
         {
             // Access temporary target under the lock held
             KisIndirectPaintingSupport::ReadLocker l(this);
 
-            // extent of m_d->selection should also be accessed under a lock,
-            // because it might be being merged in by the temporary target atm
-            effectiveExtent = effectiveSelection->selectedRect();
+            if (!paintsOutsideSelection()) {
+                // extent of m_d->selection should also be accessed under a lock,
+                // because it might be being merged in by the temporary target atm
+                QRect effectiveExtent = m_d->selection->selectedRect();
 
-            if (hasTemporaryTarget()) {
-                effectiveExtent |= temporaryTarget()->extent();
-            }
+                if (hasTemporaryTarget()) {
+                    effectiveExtent |= temporaryTarget()->extent();
+                }
 
-            if(!effectiveExtent.intersects(applyRect)) {
-                return;
+                if(!effectiveExtent.intersects(applyRect)) {
+                    return;
+                }
             }
 
             if (hasTemporaryTarget()) {
-                effectiveSelection = m_d->cachedSelection.getSelection();
-                effectiveSelection->setDefaultBounds(m_d->selection->pixelSelection()->defaultBounds());
+                effectiveSelection = m_d->safeProjection->getDeviceLazy(m_d->selection);
 
                 KisPainter::copyAreaOptimized(applyRect.topLeft(),
                                               m_d->selection->pixelSelection(),
@@ -282,31 +315,45 @@ void KisMask::apply(KisPaintDeviceSP projection, const QRect &applyRect, const Q
                 KisPainter gc(effectiveSelection->pixelSelection());
                 setupTemporaryPainter(&gc);
                 gc.bitBlt(applyRect.topLeft(), temporaryTarget(), applyRect);
+            } else {
+                m_d->safeProjection->releaseDevice();
             }
+
+            mergeInMaskInternal(projection, effectiveSelection, applyRect, needRect, maskPos);
         }
 
-        KisPaintDeviceSP cacheDevice = m_d->paintDeviceCache.getDevice(projection);
+    } else {
+        mergeInMaskInternal(projection, 0, applyRect, needRect, maskPos);
+    }
+}
 
+void KisMask::mergeInMaskInternal(KisPaintDeviceSP projection,
+                                  KisSelectionSP effectiveSelection,
+                                  const QRect &applyRect,
+                                  const QRect &preparedNeedRect,
+                                  KisNode::PositionToFilthy maskPos) const
+{
+    KisPaintDeviceSP cacheDevice = m_d->paintDeviceCache.getDevice(projection);
+
+    if (effectiveSelection) {
         QRect updatedRect = decorateRect(projection, cacheDevice, applyRect, maskPos);
 
         // masks don't have any compositioning
         KisPainter::copyAreaOptimized(updatedRect.topLeft(), cacheDevice, projection, updatedRect, effectiveSelection);
-        m_d->paintDeviceCache.putDevice(cacheDevice);
-
-        if (effectiveSelection != m_d->selection) {
-            m_d->cachedSelection.putSelection(effectiveSelection);
-        }
 
     } else {
-        KisPaintDeviceSP cacheDevice = m_d->paintDeviceCache.getDevice(projection);
-
-        cacheDevice->makeCloneFromRough(projection, needRect);
-        projection->clear(needRect);
+        cacheDevice->makeCloneFromRough(projection, preparedNeedRect);
+        projection->clear(preparedNeedRect);
 
         decorateRect(cacheDevice, projection, applyRect, maskPos);
-
-        m_d->paintDeviceCache.putDevice(cacheDevice);
     }
+
+    m_d->paintDeviceCache.putDevice(cacheDevice);
+}
+
+void KisMask::flattenSelectionProjection(KisSelectionSP selection, const QRect &dirtyRect) const
+{
+    selection->updateProjection(dirtyRect);
 }
 
 QRect KisMask::needRect(const QRect &rect,  PositionToFilthy pos) const
@@ -458,7 +505,7 @@ void KisMask::baseNodeChangedCallback()
     KisNodeSP up = parent();
     KisLayer *layer = dynamic_cast<KisLayer*>(up.data());
     if (layer) {
-        layer->notifyChildMaskChanged(this);
+        layer->notifyChildMaskChanged();
     }
     KisNode::baseNodeChangedCallback();
 }
