@@ -27,8 +27,10 @@
 #include <QVector>
 #include <QList>
 
+#include <boost/operators.hpp>
 #include "KoColorSpaceConstants.h"
 #include "KoColorConversionTransformation.h"
+#include "KoColorProofingConversionTransformation.h"
 #include "KoCompositeOp.h"
 #include <KoID.h>
 #include "kritapigment_export.h"
@@ -83,7 +85,7 @@ class KoConvolutionOp;
  * Some subclasses implement only some parts and are named Traits
  *
  */
-class KRITAPIGMENT_EXPORT KoColorSpace
+class KRITAPIGMENT_EXPORT KoColorSpace : public boost::equality_comparable<KoColorSpace>
 {
     friend class KoColorSpaceRegistry;
     friend class KoColorSpaceFactory;
@@ -107,23 +109,22 @@ public:
      * maybe convert to 3d space in future?
      */
     QPolygonF gamutXYY() const;
-    
+
     /*
      * @returns a polygon with 5 samples per channel converted to xyY, but unlike
      * gamutxyY it focuses on the luminance. This then can be used to visualise
      * the approximate trc of a given colorspace.
      */
     QPolygonF estimatedTRCXYY() const;
-    
-    QVector <qreal> colorants() const;
+
     QVector <qreal> lumaCoefficients() const;
-    
+
     //========== Channels =====================================================//
 
     /// Return a list describing all the channels this color model has. The order
     /// of the channels in the list is the order of channels in the pixel. To find
     /// out the preferred display position, use KoChannelInfo::displayPosition.
-    virtual QList<KoChannelInfo *> channels() const;
+    QList<KoChannelInfo *> channels() const;
 
     /**
      * The total number of channels for a single pixel in this color model
@@ -197,13 +198,13 @@ public:
      * it is usually the concatenation of the id of the color model and of the color
      * depth, for instance "RGBA8" or "CMYKA16" or "XYZA32f".
      */
-    virtual QString id() const;
+    QString id() const;
 
     /**
      * User visible name which contains the name of the color model and of the color depth.
-     * For intance "RGBA (8-bits)" or "CMYKA (16-bits)".
+     * For instance "RGBA (8-bits)" or "CMYKA (16-bits)".
      */
-    virtual QString name() const;
+    QString name() const;
 
     /**
      * @return a string that identify the color model (for instance "RGB" or "CMYK" ...)
@@ -249,7 +250,7 @@ public:
      * Retrieve a single composite op from the ones this colorspace offers.
      * If the requeste composite op does not exist, COMPOSITE_OVER is returned.
      */
-    virtual const KoCompositeOp * compositeOp(const QString & id) const;
+    const KoCompositeOp * compositeOp(const QString & id) const;
 
     /**
      * add a composite op to this colorspace.
@@ -304,6 +305,7 @@ public:
      * @param height in pixels
      * @param dstProfile destination profile
      * @param renderingIntent the rendering intent
+     * @param conversionFlags conversion flags
      */
     virtual QImage convertToQImage(const quint8 *data, qint32 width, qint32 height,
                                    const KoColorProfile *  dstProfile,
@@ -371,6 +373,25 @@ public:
                                  KoColorConversionTransformation::Intent renderingIntent,
                                  KoColorConversionTransformation::ConversionFlags conversionFlags) const;
 
+    virtual KoColorConversionTransformation *createProofingTransform(const KoColorSpace * dstColorSpace,
+                                                             const KoColorSpace * proofingSpace,
+                                                             KoColorConversionTransformation::Intent renderingIntent,
+                                                             KoColorConversionTransformation::Intent proofingIntent,
+                                                             KoColorConversionTransformation::ConversionFlags conversionFlags,
+                                                             quint8 *gamutWarning, double adaptationState) const;
+    /**
+     * @brief proofPixelsTo
+     * @param src source
+     * @param dst destination
+     * @param numPixels the amount of pixels.
+     * @param proofingTransform the intent used for proofing.
+     * @return
+     */
+    virtual bool proofPixelsTo(const quint8 * src,
+                               quint8 * dst,
+                               quint32 numPixels,
+                               KoColorConversionTransformation *proofingTransform) const;
+
 //============================== Manipulation functions ==========================//
 
 
@@ -466,7 +487,7 @@ public:
 
     /**
      * Get the difference between 2 colors, normalized in the range (0,255). Only completely
-     * opaque and completely transparent are taken into account when computing the different;
+     * opaque and completely transparent are taken into account when computing the difference;
      * other transparency levels are not regarded when finding the difference.
      */
     virtual quint8 difference(const quint8* src1, const quint8* src2) const = 0;
@@ -506,9 +527,9 @@ public:
     virtual void increaseGreen(quint8 * pixel, qreal step) const;
     virtual void increaseBlue(quint8 * pixel, qreal step) const;
     virtual void increaseYellow(quint8 * pixel, qreal step) const;
-    virtual void toHSY(QVector <double> channelValues, qreal *hue, qreal *sat, qreal *luma) const = 0;
+    virtual void toHSY(const QVector<double> &channelValues, qreal *hue, qreal *sat, qreal *luma) const = 0;
     virtual QVector <double> fromHSY(qreal *hue, qreal *sat, qreal *luma) const = 0;
-    virtual void toYUV(QVector <double> channelValues, qreal *y, qreal *u, qreal *v) const = 0;
+    virtual void toYUV(const QVector<double> &channelValues, qreal *y, qreal *u, qreal *v) const = 0;
     virtual QVector <double> fromYUV(qreal *y, qreal *u, qreal *v) const = 0;
     /**
      * Compose two arrays of pixels together. If source and target
@@ -517,9 +538,11 @@ public:
      * colorspace.
      *
      * @param srcSpace the colorspace of the source pixels that will be composited onto "us"
-     * @param param the information needed for blitting e.g. the source and destination pixel data,
+     * @param params the information needed for blitting e.g. the source and destination pixel data,
      *        the opacity and flow, ...
      * @param op the composition operator to use, e.g. COPY_OVER
+     * @param renderingIntent the rendering intent
+     * @param conversionFlags the conversion flags.
      *
      */
     virtual void bitBlt(const KoColorSpace* srcSpace, const KoCompositeOp::ParameterInfo& params, const KoCompositeOp* op,
@@ -530,13 +553,13 @@ public:
      * Serialize this color following Create's swatch color specification available
      * at http://create.freedesktop.org/wiki/index.php/Swatches_-_colour_file_format
      *
-     * This function doesn't create the <color /> element but rather the <CMYK />,
-     * <sRGB />, <RGB /> ... elements. It is assumed that colorElt is the <color />
+     * This function doesn't create the \<color /\> element but rather the \<CMYK /\>,
+     * \<sRGB /\>, \<RGB /\> ... elements. It is assumed that colorElt is the \<color /\>
      * element.
      *
      * @param pixel buffer to serialized
      * @param colorElt root element for the serialization, it is assumed that this
-     *                 element is <color />
+     *                 element is \<color /\>
      * @param doc is the document containing colorElt
      */
     virtual void colorToXML(const quint8* pixel, QDomDocument& doc, QDomElement& colorElt) const = 0;
@@ -546,7 +569,7 @@ public:
      * at http://create.freedesktop.org/wiki/index.php/Swatches_-_colour_file_format
      *
      * @param pixel buffer where the color will be unserialized
-     * @param elt the element to unserialize (<CMYK />, <sRGB />, <RGB />)
+     * @param elt the element to unserialize (\<CMYK /\>, \<sRGB /\>, \<RGB /\>)
      * @return the unserialize color, or an empty color object if the function failed
      *         to unserialize the color
      */
@@ -605,7 +628,11 @@ protected:
 
 inline QDebug operator<<(QDebug dbg, const KoColorSpace *cs)
 {
-    dbg.nospace() << cs->name() << " (" << cs->colorModelId().id() << "," << cs->colorDepthId().id() << " )";
+    if (cs) {
+        dbg.nospace() << cs->name() << " (" << cs->colorModelId().id() << "," << cs->colorDepthId().id() << " )";
+    } else {
+        dbg.nospace() << "0x0";
+    }
 
     return dbg.space();
 }
