@@ -43,12 +43,6 @@ KisColorSelectorSettings::KisColorSelectorSettings(QWidget *parent) :
 {
     ui->setupUi(this);
 
-    ui->lbl_lastUsedNumRows->hide();
-    ui->lastUsedColorsNumRows->hide();
-
-    ui->lbl_commonColorsNumCols->hide();
-    ui->commonColorsNumCols->hide();
-    
     resize(minimumSize());
 
     ui->colorSelectorConfiguration->setColorSpace(ui->colorSpace->currentColorSpace());
@@ -58,7 +52,7 @@ KisColorSelectorSettings::KisColorSelectorSettings(QWidget *parent) :
 
     /* color docker selector drop down */
     ui->dockerColorSettingsComboBox->addItem(i18n("Advanced Color Selector"));
-    ui->dockerColorSettingsComboBox->addItem(i18n("Color Sliders"));
+    //ui->dockerColorSettingsComboBox->addItem(i18n("Color Sliders"));
     ui->dockerColorSettingsComboBox->addItem(i18n("Color Hotkeys"));
     ui->dockerColorSettingsComboBox->setCurrentIndex(0); // start off seeing advanced color selector properties
 
@@ -106,6 +100,10 @@ KisColorSelectorSettings::KisColorSelectorSettings(QWidget *parent) :
     ui->commonColorsAlignVertical->setChecked(true);
     ui->commonColorsAlignHorizontal->setChecked(true);
     connect( ui->commonColorsAlignHorizontal, SIGNAL(toggled(bool)), this, SLOT(changedACSColorAlignment(bool)));
+    connect( ui->lastUsedColorsAlignHorizontal, SIGNAL(toggled(bool)), this, SLOT(changedACSLastUsedColorAlignment(bool)));
+
+    changedACSColorAlignment(ui->commonColorsAlignHorizontal->isChecked());
+    changedACSLastUsedColorAlignment(ui->lastUsedColorsAlignHorizontal->isChecked());
 
 
     connect(ui->colorSpace,                 SIGNAL(colorSpaceChanged(const KoColorSpace*)),
@@ -172,14 +170,12 @@ void KisColorSelectorSettings::savePreferences() const
     cfg.writeEntry("onDockerResize", ui->dockerResizeOptionsComboBox->currentIndex());
     cfg.writeEntry("zoomSelectorOptions", ui->zoomSelectorOptionComboBox->currentIndex() );
     cfg.writeEntry("zoomSize", ui->popupSize->value());
-
-
+    cfg.writeEntry("showColorSelector", ui->chkShowColorSelector->isChecked());
 
     bool useCustomColorSpace =  ui->useDifferentColorSpaceCheckbox->isChecked();
     const KoColorSpace* colorSpace = useCustomColorSpace ? ui->colorSpace->currentColorSpace() : 0;
 
-
-    KisConfig kisconfig;
+    KisConfig kisconfig(false);
     kisconfig.setCustomColorSelectorColorSpace(colorSpace);
 
     //color patches
@@ -221,7 +217,7 @@ void KisColorSelectorSettings::savePreferences() const
     cfg.writeEntry("shadeSelectorUpdateOnLeftClick", ui->shadeSelectorUpdateOnLeftClick->isChecked());
     cfg.writeEntry("shadeSelectorUpdateOnBackground", ui->shadeSelectorUpdateOnBackground->isChecked());
     cfg.writeEntry("hidePopupOnClickCheck", ui->hidePopupOnClickCheck->isChecked());
-    
+
     //mypaint model
 
     int shadeMyPaintComboBoxIndex  = ui->ACSshadeSelectorMyPaintColorModelComboBox->currentIndex();
@@ -245,15 +241,15 @@ void KisColorSelectorSettings::savePreferences() const
     //color selector
     KisColorSelectorComboBox* cstw = dynamic_cast<KisColorSelectorComboBox*>(ui->colorSelectorConfiguration);
     cfg.writeEntry("colorSelectorConfiguration", cstw->configuration().toString());
-    
+
     cfg.writeEntry("hsxSettingType", ui->colorSelectorTypeComboBox->currentIndex());
-    
+
     //luma//
     cfg.writeEntry("lumaR", ui->l_lumaR->value());
     cfg.writeEntry("lumaG", ui->l_lumaG->value());
     cfg.writeEntry("lumaB", ui->l_lumaB->value());
     cfg.writeEntry("gamma", ui->SP_Gamma->value());
-    
+
     //slider//
     hsxcfg.writeEntry("hsvH", ui->csl_hsvH->isChecked());
     hsxcfg.writeEntry("hsvS", ui->csl_hsvS->isChecked());
@@ -303,11 +299,12 @@ void KisColorSelectorSettings::changedColorDocker(int index)
         ui->colorSliderOptions->hide();
         ui->hotKeyOptions->hide();
     }
-    else if (index == 1) {  // color slider options selected
-        ui->advancedColorSelectorOptions->hide();
-        ui->hotKeyOptions->hide();
-        ui->colorSliderOptions->show();
-    } else { 
+//    else if (index == 1) {  // color slider options selected
+//        ui->advancedColorSelectorOptions->hide();
+//        ui->hotKeyOptions->hide();
+//        ui->colorSliderOptions->show();
+//    }
+    else {
        ui->colorSliderOptions->hide();
        ui->advancedColorSelectorOptions->hide();
        ui->hotKeyOptions->show();
@@ -319,10 +316,10 @@ void KisColorSelectorSettings::changedACSColorSelectorType(int index)
     ui->lumaCoefficientGroupbox->setVisible(false);
 
     if (index == 0)     {  // HSV
-        ui->ACSTypeDescriptionLabel->setText(i18n("Values goes from black to white, or black to the most saturated colour. Saturation, in turn, goes from the most saturated colour to white, grey or black."));
+        ui->ACSTypeDescriptionLabel->setText(i18n("Values goes from black to white, or black to the most saturated color. Saturation, in turn, goes from the most saturated color to white, gray or black."));
     }
     else if (index == 1)     {  // HSL
-        ui->ACSTypeDescriptionLabel->setText(i18n("Lightness goes from black to white, with middle grey being equal to the most saturated colour."));
+        ui->ACSTypeDescriptionLabel->setText(i18n("Lightness goes from black to white, with middle gray being equal to the most saturated color."));
     }
     else if (index == 2)     {  // HSI
         ui->ACSTypeDescriptionLabel->setText(i18n("Intensity maps to the sum of rgb components"));
@@ -343,28 +340,24 @@ void KisColorSelectorSettings::changedACSColorAlignment(bool toggled)
     // this slot is tied to the horizontal radio button's state being changed
     // you can infer the vertical state
 
-    if (toggled) {  // horizontal layout. show rows
-        ui->lbl_commonColorsNumCols->hide();
-        ui->commonColorsNumCols->hide();
-
-        ui->lbl_commonColorsNumRows->show();
-        ui->commonColorsNumRows->show();
-
-    } else {
-        ui->lbl_commonColorsNumCols->show();
-        ui->commonColorsNumCols->show();
-
-        ui->lbl_commonColorsNumRows->hide();
-        ui->commonColorsNumRows->hide();
-
-    }
-
-
-
+    ui->lbl_commonColorsNumCols->setDisabled(toggled);
+    ui->commonColorsNumCols->setDisabled(toggled);
 
+    ui->lbl_commonColorsNumRows->setEnabled(toggled);
+    ui->commonColorsNumRows->setEnabled(toggled);
 }
 
+void KisColorSelectorSettings::changedACSLastUsedColorAlignment(bool toggled)
+{
+    // this slot is tied to the horizontal radio button's state being changed
+    // you can infer the vertical state
 
+    ui->lbl_lastUsedNumCols->setDisabled(toggled);
+    ui->lastUsedColorsNumCols->setDisabled(toggled);
+
+    ui->lbl_lastUsedNumRows->setEnabled(toggled);
+    ui->lastUsedColorsNumRows->setEnabled(toggled);
+}
 
 void KisColorSelectorSettings::changedACSShadeSelectorType(int index)
 {
@@ -384,14 +377,8 @@ void KisColorSelectorSettings::changedACSShadeSelectorType(int index)
         ui->myPaintColorModelLabel->hide();
         ui->ACSshadeSelectorMyPaintColorModelComboBox->hide();
     }
-
-
-
 }
 
-
-
-
 void KisColorSelectorSettings::useDifferentColorSpaceChecked(bool enabled)
 {
     ui->colorSpace->setEnabled(enabled);
@@ -411,13 +398,13 @@ void KisColorSelectorSettings::loadPreferences()
     ui->dockerResizeOptionsComboBox->setCurrentIndex(  (int)cfg.readEntry("onDockerResize", 0) );
     ui->zoomSelectorOptionComboBox->setCurrentIndex(   (int) cfg.readEntry("zoomSelectorOptions", 0) );
     ui->popupSize->setValue(cfg.readEntry("zoomSize", 280));
-
+    ui->chkShowColorSelector->setChecked((bool) cfg.readEntry("showColorSelector", true));
 
     {
-        KisConfig kisconfig;
+        KisConfig kisconfig(true);
         const KoColorSpace *cs = kisconfig.customColorSelectorColorSpace();
 
-        if(cs) {
+        if (cs) {
             ui->useDifferentColorSpaceCheckbox->setChecked(true);
             ui->colorSpace->setEnabled(true);
             ui->colorSpace->setCurrentColorSpace(cs);
@@ -444,7 +431,7 @@ void KisColorSelectorSettings::loadPreferences()
     a = cfg.readEntry("commonColorsAlignment", false);
     ui->commonColorsAlignVertical->setChecked(a);
     ui->commonColorsAlignHorizontal->setChecked(!a);
-    ui->commonColorsAllowScrolling->setChecked(cfg.readEntry("commonColorsScrolling", false));
+    ui->commonColorsAllowScrolling->setChecked(cfg.readEntry("commonColorsScrolling", true));
     ui->commonColorsNumCols->setValue(cfg.readEntry("commonColorsNumCols", 1));
     ui->commonColorsNumRows->setValue(cfg.readEntry("commonColorsNumRows", 1));
     ui->commonColorsPatchCount->setValue(cfg.readEntry("commonColorsCount", 12));
@@ -453,7 +440,7 @@ void KisColorSelectorSettings::loadPreferences()
     ui->commonColorsAutoUpdate->setChecked(cfg.readEntry("commonColorsAutoUpdate", false));
 
     //shade selector
-    QString shadeSelectorType=cfg.readEntry("shadeSelectorType", "MyPaint");
+    QString shadeSelectorType=cfg.readEntry("shadeSelectorType", "Minimal");
 
     if ( shadeSelectorType == "MyPaint") {
         ui->ACSShadeSelectorTypeComboBox->setCurrentIndex(0);
@@ -496,14 +483,14 @@ void KisColorSelectorSettings::loadPreferences()
 
     //color selector
     KisColorSelectorComboBox* cstw = dynamic_cast<KisColorSelectorComboBox*>(ui->colorSelectorConfiguration);
-    cstw->setConfiguration(KisColorSelector::Configuration::fromString(cfg.readEntry("colorSelectorConfiguration", "3|0|5|0"))); // triangle selector
-    
+    cstw->setConfiguration(KisColorSelectorConfiguration::fromString(cfg.readEntry("colorSelectorConfiguration", "3|0|5|0"))); // triangle selector
+
     //luma values//
     ui->l_lumaR->setValue(cfg.readEntry("lumaR", 0.2126));
     ui->l_lumaG->setValue(cfg.readEntry("lumaG", 0.7152));
     ui->l_lumaB->setValue(cfg.readEntry("lumaB", 0.0722));
     ui->SP_Gamma->setValue(cfg.readEntry("gamma", 2.2));
-    
+
     //color sliders//
     ui->csl_hsvH->setChecked(hsxcfg.readEntry("hsvH", false));
     ui->csl_hsvS->setChecked(hsxcfg.readEntry("hsvS", false));
@@ -525,7 +512,7 @@ void KisColorSelectorSettings::loadPreferences()
     ui->sb_rg->setValue(hotkeycfg.readEntry("steps_redgreen", 10));
     ui->sb_by->setValue(hotkeycfg.readEntry("steps_blueyellow", 10));
 
-    
+
 }
 
 void KisColorSelectorSettings::loadDefaultPreferences()
@@ -537,6 +524,7 @@ void KisColorSelectorSettings::loadDefaultPreferences()
     ui->dockerResizeOptionsComboBox->setCurrentIndex(0);
     ui->zoomSelectorOptionComboBox->setCurrentIndex(0);
     ui->popupSize->setValue(280);
+    ui->chkShowColorSelector->setChecked(true);
 
 
     ui->useDifferentColorSpaceCheckbox->setChecked(false);
@@ -558,7 +546,7 @@ void KisColorSelectorSettings::loadDefaultPreferences()
     ui->commonColorsShow->setChecked(true);
     ui->commonColorsAlignVertical->setChecked(false);
     ui->commonColorsAlignHorizontal->setChecked(true);
-    ui->commonColorsAllowScrolling->setChecked(false);
+    ui->commonColorsAllowScrolling->setChecked(true);
     ui->commonColorsNumCols->setValue(1);
     ui->commonColorsNumRows->setValue(1);
     ui->commonColorsPatchCount->setValue(12);
@@ -567,7 +555,7 @@ void KisColorSelectorSettings::loadDefaultPreferences()
     ui->commonColorsAutoUpdate->setChecked(false);
 
     //shade selector
-    ui->ACSShadeSelectorTypeComboBox->setCurrentIndex(0); // MyPaint
+    ui->ACSShadeSelectorTypeComboBox->setCurrentIndex(1); // Minimal
 
     ui->ACSshadeSelectorMyPaintColorModelComboBox->setCurrentIndex(0);
 
@@ -586,16 +574,16 @@ void KisColorSelectorSettings::loadDefaultPreferences()
 
     // set advanced color selector to use HSV
     ui->colorSelectorTypeComboBox->setCurrentIndex(0);
-    
+
     KisColorSelectorComboBox* cstw = dynamic_cast<KisColorSelectorComboBox*>(ui->colorSelectorConfiguration);
-    cstw->setConfiguration(KisColorSelector::Configuration("3|0|5|0")); // triangle selector
-    
+    cstw->setConfiguration(KisColorSelectorConfiguration("3|0|5|0")); // triangle selector
+
     //luma//
     ui->l_lumaR->setValue(0.2126);
     ui->l_lumaG->setValue(0.7152);
     ui->l_lumaB->setValue(0.0722);
     ui->SP_Gamma->setValue(2.2);
-    
+
     //color sliders//
     ui->csl_hsvH->setChecked(false);
     ui->csl_hsvS->setChecked(false);
