@@ -57,7 +57,10 @@ void KisToolEllipse::finishRect(const QRectF& rect, qreal roundCornersX, qreal r
     if (rect.isEmpty() || !blockUntilOperationsFinished())
         return;
 
-    if (!currentNode()->inherits("KisShapeLayer")) {
+    const KisToolShape::ShapeAddInfo info =
+        shouldAddShape(currentNode());
+
+    if (!info.shouldAddShape) {
         KisFigurePaintingToolHelper helper(kundo2_i18n("Draw Ellipse"),
                                            image(),
                                            currentNode(),
@@ -70,6 +73,9 @@ void KisToolEllipse::finishRect(const QRectF& rect, qreal roundCornersX, qreal r
         KoShape* shape = KisShapeToolHelper::createEllipseShape(r);
         KoShapeStrokeSP border(new KoShapeStroke(currentStrokeWidth(), currentFgColor().toQColor()));
         shape->setStroke(border);
+
+        info.markAsSelectionShapeIfNeeded(shape);
+
         addShape(shape);
     }
     notifyModified();
